@@ -40,10 +40,10 @@ Source code site: https://github.com/abarker/pdfCropMargins
 # is slightly different.
 #
 # This program (like the Ghostscript program) uses the PDF ordering convention
-# (lbrt) for listing margins and defining boxes.  Note that PIL uses some
+# (lbrt) for listing margins and defining boxes.  Note that Pillow uses some
 # different conventions.  The origin in PDFs is the lower left going up but the
-# origin in PIL images is the upper left going down.  Also, the bounding box
-# routine of PIL returns ltrb instead of lbrt.  Keep in mind that the program
+# origin in Pillow images is the upper left going down.  Also, the bounding box
+# routine of Pillow returns ltrb instead of lbrt.  Keep in mind that the program
 # needs to make these conversions when rendering explicitly to images.
 
 from __future__ import print_function, division, absolute_import
@@ -51,11 +51,15 @@ import sys
 import os
 import shutil
 import time
-import readline # Makes prompts go to stdout rather than stderr.
+try:
+    import readline # Makes prompts go to stdout rather than stderr.
+except ImportError: # Not available on Windows.
+    pass
 
 from . import __version__ # Get the version number from the __init__.py file.
 from .manpage_data import cmd_parser, DEFAULT_THRESHOLD_VALUE
 from .prettified_argparse import parse_command_line_arguments
+from .pymupdf_routines import has_mupdf
 
 from . import external_program_calls as ex
 project_src_directory = ex.project_src_directory
@@ -66,9 +70,10 @@ try:
                                 FloatObject, IndirectObject)
     from PyPDF2.utils import PdfReadError
 except ImportError:
-    print("\nError in pdfCropMargins: No system pyPdf Python package"
-          " was found.\n", file=sys.stderr)
-    raise
+    print("\nError in pdfCropMargins: No system PyPDF2 Python package"
+          "\nwas found.  Reinstall pdfCropMargins via pip or install that"
+          "\ndependency ('pip install pypdf2').\n", file=sys.stderr)
+    ex.cleanup_and_exit(1)
 
 from .calculate_bounding_boxes import get_bounding_box_list
 
@@ -119,11 +124,15 @@ def parse_page_range_specifiers(spec_string, all_page_nums):
             left_arg = int(split_range[0])-1
             right_arg = int(split_range[1])
             if left_arg >= right_arg:
-                print("Error in pdfCropMargins: left argument of range cannot be less"
-                      " than the right one.", file=sys.stderr)
+                print("Error in pdfCropMargins: Left argument of page range '{}' cannot"
+                      " be less than the right one.".format(spec_string), file=sys.stderr)
                 raise ValueError
             page_nums_to_crop.update(set(range(left_arg, right_arg)))
     page_nums_to_crop = page_nums_to_crop & all_page_nums # intersect chosen with actual
+    if not page_nums_to_crop: # Empty set of pages.
+        print("Error in pdfCropMargins: Page range selection '{}' results in empty"
+                " set.".format(spec_string), file=sys.stderr)
+        raise ValueError
     return page_nums_to_crop
 
 def parse_page_ratio_argument(ratio_arg):
@@ -880,14 +889,29 @@ def process_command_line_arguments(parsed_args):
                 ex.cleanup_and_exit(1)
 
     #
-    # Process options dealing with external programs.
+    # Process options dealing with rendering and external programs.
     #
 
-    if args.gsBbox and len(args.fullPageBox) > 1:
-        print("\nWarning: only one --fullPageBox value can be used with the -gs option.",
-              "\nIgnoring all but the first one.", file=sys.stderr)
+    if args.gsRender:
+        args.calcbb = "gr" # Backward compat.
+    if args.gsBbox:
+        args.calcbb = "gb" # Backward compat.
+    if args.calcbb == "m" and not has_mupdf:
+        print("Error in pdfCropMargins: The option '--calcbb m' was selected"
+              "\nbut PyMuPDF (at least v1.14.5) was not installed in Python."
+              "\nInstalling pdfCropMargins with the GUI option will include that"
+              "\ndependency.", file=sys.stderr)
+        ex.cleanup_and_exit(1)
+    if args.calcbb == "d" and has_mupdf:
+        args.calcbb = "m" # Default to PyMuPDF.
+    elif args.calcbb == "d": # Rendering without PyMuPDF.
+        args.calcbb = "o"     # Revert to old method.
+
+    if args.calcbb == "gb" and len(args.fullPageBox) > 1:
+        print("\nWarning: only one --fullPageBox value can be used with the '--calcbb gb'"
+              "\nor '--gsBbox' option. Ignoring all but the first one.", file=sys.stderr)
         args.fullPageBox = [args.fullPageBox[0]]
-    elif args.gsBbox and not args.fullPageBox:
+    elif args.calcbb == "gb" and not args.fullPageBox:
         args.fullPageBox = ["c"] # gs default
     elif not args.fullPageBox:
         args.fullPageBox = ["m", "c"] # usual default
@@ -902,30 +926,37 @@ def process_command_line_arguments(parsed_args):
     if args.ghostscriptPath:
         ex.set_gs_executable_to_string(args.ghostscriptPath)
 
-    # If the option settings require pdftoppm, make sure we have a running
-    # version.  If '--gsBbox' isn't chosen then assume that PDF pages are to be
-    # explicitly rendered.  In that case we either need pdftoppm or gs to do the
-    # rendering.
+    # If the option settings require pdftoppm, make sure we have a running version.
     gs_render_fallback_set = False # Set True if we switch to gs option as a fallback.
-    if not args.gsBbox and not args.gsRender:
+    if args.calcbb in ["p", "o"]:
+        # Note that after this block, the `--calcbb o` option is converted to 'p' or 'gr'.
         found_pdftoppm = ex.init_and_test_pdftoppm_executable(
                                                    prefer_local=args.pdftoppmLocal)
         if args.verbose:
             print("\nFound pdftoppm program at:", found_pdftoppm)
-        if not found_pdftoppm:
-            args.gsRender = True
+        if found_pdftoppm:
+            args.calcbb = "p"
+        elif args.calcbb == "p":
+                print("\nError in pdfCropMargins: The '--calcbb p' option was specified "
+                      "\nbut the pdftoppm executable could not be located.  Is it"
+                      "\ninstalled and in the PATH for command execution?\n",
+                      file=sys.stderr)
+                ex.cleanup_and_exit(1)
+        else:
+            # Try fallback to gs.
             gs_render_fallback_set = True
+            args.calcbb = "gr"
             if args.verbose:
                 print("\nNo pdftoppm executable found; using Ghostscript for rendering.")
 
     # If any options require Ghostscript, make sure it is installed.
-    if args.gsBbox or args.gsFix or args.gsRender:
+    if args.calcbb == "gr" or args.calcbb == "gb" or args.gsFix:
         found_gs = ex.init_and_test_gs_executable()
         if args.verbose:
             print("\nFound Ghostscript program at:", found_gs)
-    if args.gsBbox and not found_gs:
-        print("\nError in pdfCropMargins: The '--gsBbox' option was specified but"
-              "\nthe Ghostscript executable could not be located.  Is it"
+    if args.calcbb == "gb" and not found_gs:
+        print("\nError in pdfCropMargins: The '--calcbb gb' or '--gsBbox' option was"
+              "\nspecified but the Ghostscript executable could not be located.  Is it"
               "\ninstalled and in the PATH for command execution?\n", file=sys.stderr)
         ex.cleanup_and_exit(1)
     if args.gsFix and not found_gs:
@@ -933,27 +964,30 @@ def process_command_line_arguments(parsed_args):
               "\nthe Ghostscript executable could not be located.  Is it"
               "\ninstalled and in the PATH for command execution?\n", file=sys.stderr)
         ex.cleanup_and_exit(1)
-    if args.gsRender and not found_gs:
+    if (args.calcbb == "gr" or gs_render_fallback_set) and not found_gs:
         if gs_render_fallback_set:
             print("\nError in pdfCropMargins: Neither Ghostscript nor pdftoppm"
                   "\nwas found in the PATH for command execution.  At least one is"
-                  "\nrequired.\n", file=sys.stderr)
+                  "\nrequired without PyMuPDF installed.\n", file=sys.stderr)
         else:
-            print("\nError in pdfCropMargins: The '--gsRender' option was specified but"
-                  "\nthe Ghostscript executable could not be located.  Is it"
-                  "\ninstalled and in the PATH for command execution?\n", file=sys.stderr)
+            print("\nError in pdfCropMargins: The '--calcbb gr' or the '--gsRender' option"
+                  "\nwas specified but the Ghostscript executable could not be located.  Is "
+                  "\nit installed and in the PATH for command execution?\n", file=sys.stderr)
         ex.cleanup_and_exit(1)
 
     # Give a warning message if incompatible option combinations have been selected.
-    if args.threshold[0] != DEFAULT_THRESHOLD_VALUE and args.gsBbox:
+    if args.threshold[0] != DEFAULT_THRESHOLD_VALUE and args.calcbb == "gb":
             print("\nWarning in pdfCropMargins: The '--threshold' option is ignored"
-                "\nwhen the '--gsBbox' option is also selected.\n", file=sys.stderr)
-    if args.gsBbox and args.numBlurs:
+                "\nwhen the '--calcbb gb' or '--gsBbox' option is also selected.\n",
+                file=sys.stderr)
+    if args.calcbb == "gb" and args.numBlurs:
         print("\nWarning in pdfCropMargins: The '--numBlurs' option is ignored"
-              "\nwhen the '--gsBbox' option is also selected.\n", file=sys.stderr)
-    if args.gsBbox and args.numSmooths:
+              "\nwhen the '--calcbb gb' or '--gsBbox' option is also selected.\n",
+              file=sys.stderr)
+    if args.calcbb == "gb" and args.numSmooths:
         print("\nWarning in pdfCropMargins: The '--numSmooths' option is ignored"
-              "\nwhen the '--gsBbox' option is also selected.\n", file=sys.stderr)
+              "\nwhen the '--calcbb gb' or '--gsBbox' option is also selected.\n",
+              file=sys.stderr)
 
     if args.gsFix:
         if args.verbose:
